__all__ = ['BlackOnWhite', 'NameConfirm', 'OnscreenText', 'Plain', 'ScreenPrompt', 'ScreenTitle']

from typing import Any, Final, Literal, overload
from typing_extensions import TypeAlias, deprecated

from direct._typing import Unused
from panda3d._typing import Vec4Like
from panda3d.core import LColor, NodePath, TextFont, TextNode

_Alignment: TypeAlias = Literal[0, 1, 2, 3, 4, 5]
_OrderedPair: TypeAlias = tuple[float, float]

Plain: Final = 1
ScreenTitle: Final = 2
ScreenPrompt: Final = 3
NameConfirm: Final = 4
BlackOnWhite: Final = 5

class OnscreenText(NodePath[TextNode]):
    textNode: TextNode
    mayChange: bool
    isClean: bool
    decal: bool
    font: TextFont
    text: str
    text_pos: _OrderedPair
    text_r: float
    roll: float
    text_scale: _OrderedPair
    scale: _OrderedPair
    wordwrap: float
    fg: LColor
    bg: LColor
    shadow: LColor
    frame: LColor
    align: _Alignment
    unicodeText: bool
    def __init__(
        self,
        text: str = '',
        style: int = 1,
        pos: _OrderedPair = (0, 0),
        roll: float = 0,
        scale: float | _OrderedPair | None = None,
        fg: Vec4Like | None = None,
        bg: Vec4Like | None = None,
        shadow: Vec4Like | None = None,
        shadowOffset: _OrderedPair = (0.04, 0.04),
        frame: Vec4Like | None = None,
        align: _Alignment | None = None,
        wordwrap: float | None = None,
        drawOrder: int | None = None,
        decal: bool = ...,
        font: TextFont | None = None,
        parent: NodePath | None = None,
        sort: int = 0,
        mayChange: bool = True,
        direction: Literal['LTR', 'RTL', 'ltr', 'rtl'] | None = None,
    ) -> None: ...
    def cleanup(self) -> None: ...
    def destroy(self) -> None: ...
    def freeze(self) -> None: ...
    def thaw(self) -> None: ...
    def setDecal(self, decal: bool) -> None: ...
    def getDecal(self) -> bool: ...
    def setFont(self, font: TextFont) -> None: ...
    def getFont(self) -> TextFont: ...
    def clearText(self) -> None: ...
    def setText(self, text: str) -> None: ...
    def appendText(self, text: str) -> None: ...
    def getText(self) -> str: ...
    def setTextX(self, x: float) -> None: ...
    @deprecated('Use `.setTextX()` method instead.')
    def setX(self, x: float) -> None: ...  # type: ignore[override]
    def setTextY(self, y: float) -> None: ...
    @deprecated('Use `.setTextY()` method instead.')
    def setY(self, y: float) -> None: ...  # type: ignore[override]
    @overload
    def setTextPos(self, x: float, y: float) -> None: ...
    @overload
    def setTextPos(self, x: _OrderedPair, y: None = None) -> None: ...
    def getTextPos(self) -> _OrderedPair: ...
    @deprecated('Use `.setTextPos()` method or `.text_pos` property instead.')
    def setPos(self, x: float, y: float) -> None: ...  # type: ignore[override]
    @deprecated('Use `.getTextPos()` method or `.text_pos` property instead.')
    def getPos(self) -> _OrderedPair: ...  # type: ignore[override]
    @property
    def pos(self) -> _OrderedPair: ...
    def setTextR(self, r: float) -> None: ...
    def getTextR(self) -> float: ...
    @deprecated('Use ``setTextR(-roll)`` instead (note the negated sign).')
    def setRoll(self, roll: float) -> None: ...
    @deprecated('Use ``-getTextR()`` instead (note the negated sign).')
    def getRoll(self) -> float: ...
    @overload
    def setTextScale(self, sx: float | _OrderedPair, sy: None = None) -> None: ...
    @overload
    def setTextScale(self, sx: float, sy: float) -> None: ...
    def getTextScale(self) -> _OrderedPair: ...
    @overload  # type: ignore[override]
    @deprecated('Use `.setTextScale()` method or `.text_scale` property instead.')
    def setScale(self, sx: float | _OrderedPair, sy: None = None) -> None: ...
    @overload
    @deprecated('Use `.setTextScale()` method or `.text_scale` property instead.')
    def setScale(self, sx: float, sy: float) -> None: ...
    @deprecated('Use `.getTextScale()` method or `.text_scale` property instead.')
    def getScale(self) -> _OrderedPair: ...  # type: ignore[override]
    def updateTransformMat(self) -> None: ...
    def setWordwrap(self, wordwrap: float) -> None: ...
    def getWordwrap(self) -> float: ...
    def setFg(self, fg: Vec4Like) -> None: ...
    def setBg(self, bg: Vec4Like) -> None: ...
    def setShadow(self, shadow: Vec4Like) -> None: ...
    def setFrame(self, frame: Vec4Like) -> None: ...
    def configure(self, option: Unused = None, **kw: Any) -> None: ...
    def __setitem__(self, key: str, value: Any) -> None: ...
    def setAlign(self, align: float) -> None: ...
    def __getitem__(self, option: str) -> Any: ...
    cget = __getitem__
